import es

selfmodule = __import__(__name__)

class Syn_Event_Manager(object):
    ''' Event manager class '''
    def __init__(self):
        self.eventhandlers = {}
    def registerForEvent(self, EventName, EventDef):
        if EventName in self.eventhandlers:
            return self.eventhandlers[EventName].add(EventDef)
        return False
    def unregisterForEvent(self, EventName, EventDef):
        if EventName in self.eventhandlers:
            return self.eventhandlers[EventName].remove(EventDef)
        return False
    def registerEvent(self, EventName, handler):
        self.eventhandlers[EventName] = handler(EventName)
    def unregisterEvent(self, EventName):
        del self.eventhandlers[EventName]

class Syn_Event_Info(object):
    ''' Event information class '''
    variables = ['es_event']
    types = ['string']
    def __setitem__(self, var, typ):
        self.variables.append(var)
        self.types.append(typ)
    def __iter__(self):
        self.iterlocation = 0
        return self
    def next(self):
        if self.iterlocation < len(self.variables):
            obj = (self.variables[self.iterlocation], self.types[self.iterlocation])
            self.iterlocation += 1
            return obj
        raise StopIteration

class base_event(object):
    '''
Base event object
    '''
    eventinfo = Syn_Event_Info()
    eventlisteners = set()
    def __init__(self, EventName):
        self.eventname = EventName
        self.classname = self.__class__.__name__
        self.createEventInfo()
    def add(self, EventDef):
        if EventDef in self.eventlisteners:
            return False
        self.eventlisteners.add(EventDef)
        if len(self.eventlisteners) == 1:
            self.activateListener()
        return True
    def remove(self, EventDef):
        if EventDef not in self.eventlisteners:
            return False
        self.eventlisteners.remove(EventDef)
        if len(self.eventlisteners) == 0:
            self.deactivateListener()
        return True
    def activateListener(self):
        es.addons.registerForEvent(selfmodule, self.classname, self.listener)
    def deactivateListener(self):
        es.addons.unregisterForEvent(selfmodule, self.classname)
    def listener(self, ev):
        event_var = {'es_event': self.eventname}
        self.createEventVars(event_var, ev)
        for l in self.eventlisteners:
            l(event_var)
    def createEventVars(self, new_ev, old_ev):
        return False
    def createEventInfo(self):
        raise NotImplementedError, "createEventInfo not implemented"
    def attachEventInfo(self, new_ev, *args):
        argiter = iter(args)
        for evinfo in self.eventinfo:
            ev_name = evinfo[0]
            if ev_name not in new_ev:
                new_ev[ev_name] = argiter.next()


class player_death(base_event):
    ''' Player death event for Synergy '''
    def createEventInfo(self):
        self.eventinfo['userid'] = 'short'
        self.eventinfo['attacker'] = 'short'
        self.eventinfo['victim'] = 'short'
        self.eventinfo['killer'] = 'short'
        self.eventinfo['weapon'] = 'string'
    def createEventVars(self, new_ev, old_ev):
        userid = 0
        attacker = 0
        victim = int(old_ev['victim'])
        killer = int(old_ev['killer'])
        weapon = str(old_ev['weapon'])
        players = es.createplayerlist()
        for uid in players:
            if players[uid]['index'] == victim:
                userid = uid
            if players[uid]['index'] == killer:
                attacker = uid
        del players
        self.attachEventInfo(new_ev, userid, attacker, victim, killer, weapon)
